!debug = 0 			'turn on debugging

logmode +addin
if !debug = 1 then
	logmode +debug
else 
	setmaxerrs 1000
endif

%vars = ""    'variables to use in VAR
%smpl = @pagesmpl     'sample over which to estimate
%lags = "3"				'number of lags
%lambda0 = "0.8"		'prior tightness
%lambda1 = "0.1"		'AR(1) prior tightness
%lambda3 = "1"		'lag decay
%lambda4 = "0.1"		'intercept tightness
%mu5 = "2"				'sum of coefficient prior weight
%mu6 = "5"				'drift prior
%prior = "0"				'prior distribution  (0 = Normal/Wishart, 1 = Normal=flat)
%qm = "4"				'frequency of the data for lag decay.
!doIRF=1				'perform impulse responses
%IRFPers=@str(2*@val(%lags))	'number of periods for IRF.
!doGUI = 1				'show dialog.

%origsmpl = @pagesmpl
'check page frequency is allowed
%freq = @pagefreq
if %freq<>"A" and %freq<>"M" and %freq<>"Q" and %freq<>"U" then
	@uiprompt("Invalid frequency - this procedure requires annual, monthly, quarterly or undated data")
	stop
endif


'process arguments
if @len(%args) > 0 then
	%lags = %0  'lags is the first argument
	%vars = @wmid(%args,2)  'variables are the second argument onwards
	if @len(%vars) > 0 and @len(%lags) > 0 then
		!dogui = 0
	endif
endif

'process options
if @len(@option(1)) > 0 then
	%temp = @equaloption("L0")
	if @len(%temp)>0 then
		%lambda0 = %temp
	endif
	%temp = @equaloption("L1")
	if @len(%temp)>0 then
		%lambda1 = %temp
	endif
	%temp = @equaloption("L3")
	if @len(%temp)>0 then
		%lambda3 = %temp
	endif
	%temp = @equaloption("L4")
	if @len(%temp)>0 then
		%lambda4 = %temp
	endif
	%temp = @equaloption("Mu5")
	if @len(%temp)>0 then
		%mu5 = %temp
	endif
	%temp = @equaloption("Mu6")
	if @len(%temp)>0 then
		%mu6 = %temp
	endif
	%temp = @equaloption("QM")
	if @len(%temp)>0 then
		%qm = %temp
	endif
	%temp = @equaloption("prior")
	if @len(%temp)>0 then
		%prior = %temp
	endif
	%temp = @equaloption("irfper")
	if @len(%temp)>0 then
		!doIRF = 1 
		%IRFPers = %temp
	else
		!doIRF = 0
	endif
endif



'throw up dialog
%vars = @upper(%vars)
%plist = "Normal-Wishart Normal-flat"
!prior = @val(%prior) + 1
if !doGUI=1 then
	!result = @uidialog("caption","Bayesian VAR estimation",  _
						"edit",%vars,"Enter a list of endogenous variables", _
						"edit", %lags,"Enter a number of lags",  _
						"list",!prior,"Prior",%plist,  _
						"edit",%smpl,"Enter an estimation sample", _
						"check",!doIRF,"Calculate impulse responses", _
						"edit", %IRFPers,"Enter a number of periods for IRFs", _
						"colbreak", _
						"edit",%lambda0,"Lambda 0 - Overall prior tightness",  _
						"edit",%lambda1,"Lambda 1 - AR(1) tightness",  _
						"edit",%lambda3,"Lambda 3 - Lag decay",  _
						"edit",%lambda4,"Lambda 4 - Intercept tightness",  _
						"edit",%mu5,"Mu 5 - Sum of coefficient prior weight", _
						"edit",%mu6, "Mu 6 - Drift prior", _
						"edit", %qm, "QM - Lag decay frequency" _
						)
	if !result = -1 then
		stop
	endif
endif
%prior = @str(!prior-1)

smpl {%smpl}
if @lasterrnum > 0 then
	@uiprompt("Invalid sample")
	stop
endif

'validate options
if @wcount(%vars) = 0 then
	@uiprompt("Please enter a list of endogenous variables")
	stop
endif
if @val(%lambda0) = NA or @val(%lambda0) < 0 or @val(%lambda0) > 1 then
	@uiprompt("Invalid value for Lambda 0.  Please enter a number between 0 and 1")
	stop
endif
if @val(%lambda1) = NA or @val(%lambda1) < 0 or @val(%lambda1) > 1 then
	@uiprompt("Invalid value for Lambda 1.  Please enter a number between 0 and 1")
	stop
endif
if @val(%lambda3) = NA or @val(%lambda3) < 0  then
	@uiprompt("Invalid value for Lambda 3.  Please enter a number greater than 0")
	stop
endif
if @val(%lambda4) = NA or @val(%lambda4) < 0  then
	@uiprompt("Invalid value for Lambda 4.  Please enter a number greater than 0")
	stop
endif
if @val(%Mu5) = NA or @val(%Mu5) < 0  then
	@uiprompt("Invalid value for Mu 5.  Please enter a number greater than 0")
	stop
endif
if @val(%Mu6) = NA or @val(%Mu6) < 0  then
	@uiprompt("Invalid value for Mu 6.  Please enter a number greater than 0")
	stop
endif
if @val(%lags) = NA or @val(%lags) < 0 or @floor(@val(%lags))<>@val(%lags) then
	@uiprompt("Invalid number of lags, please enter a positive integer")
	stop
endif
if @val(%qm) = NA or @val(%qm) < 0 or @floor(@val(%qm))<>@val(%qm) then
	@uiprompt("Invalid number for QM, please enter a positive integer")
	stop
endif
if !doirf=1 then
	if @val(%irfpers) = na or @val(%irfpers)<=0 or @floor(@val(%irfpers))<>@val(%irfpers) then
		@uiprompt("Invalid value for IRF periods.  Please enter an integer greater than 0")
		stop
	endif
endif

'set sample to remove NAs
%grp= @getnextname("_g")
group {%grp} {%vars}
%smplser = @getnextname("_ser")
series {%smplser} = 1
smpl if {%smplser} = 1 and @robs({%grp}) = @max(@robs({%grp}))

'open up connection to R
if !debug = 1 then
	xopen(r, case=lower)
else 
	xopen(r, nolog, case=lower) 
endif
%str = @lasterrstr
if (@instr(%str,"StatConnectorSrv")>0) then 
	@uiprompt("R is not installed, please check your EViews documentation for instructions on installing R.")
	smpl {%origsmpl}
	d(noerr) {%cn} {%covn} {%llfn} {%mpostn} {%cpostn} {%tn} {%irftn} {%graphs} {%gname} {%grp} {%smplser}
	stop
endif

' install and load the MSBVAR package in R
xrun "if (sum(.packages(TRUE)==""MSBVAR"")==0) install.packages(""MSBVAR"")"
xrun "library(MSBVAR)"

'put names across
%varl=""
for !i=1 to @wcount(%vars)
	%var = @word(%vars,!i)
	%err = %var + " is not a valid series object. Note auto-series are not allowed in this procedure"
	if @isobject(%var) = 0 then
		@uiprompt(%err)
		d(noerr) {%cn} {%covn} {%llfn} {%mpostn} {%cpostn} {%tn} {%irftn} {%graphs} {%gname} {%grp} {%smplser}
		smpl {%origsmpl}
		stop
	endif
	if {%var}.@type <> "SERIES" then
		@uiprompt(%err)
		d(noerr) {%cn} {%covn} {%llfn} {%mpostn} {%cpostn} {%tn} {%irftn} {%graphs} {%gname} {%grp} {%smplser}
		smpl {%origsmpl}
		stop
	endif
	xput(rtype=ts) {%var}
	%varl = %varl + @xputnames + ", "
next
%varl = @stripcommas(%varl)

'build a TS object from the individual series sent across
%cmd = "g.vars = cbind(" + @lower(%varl) + ")"
xrun %cmd
if @lasterrnum>0 then
	@uiprompt("R reported an error - possibly this is because you have internal NAs in your data - internal NAs are not allowed, or a non-continuous sample.")
	d(noerr) {%cn} {%covn} {%llfn} {%mpostn} {%cpostn} {%tn} {%irftn} {%graphs} {%gname} {%grp} {%smplser}
	smpl {%origsmpl}
	stop
endif

'-------------------------------------------------------------------------------------------------------------Estimation

'estimate BVAR
!ML = @val(%lags)
%cmd =  "BVAR <- szbvar(g.vars, p=" + %lags + ", z=NULL, lambda0=" + %lambda0 + ", lambda1=" + %lambda1 + ", lambda3=" + %lambda3 + ",lambda4=" + %lambda4 + ", lambda5=0.07, mu5=" + %mu5 + ", mu6=" + %mu6 + ", qm=" + %qm + ", prior=" + %prior + ", posterior.fit=TRUE)"
xrun %cmd

'retrieve coefs and the covariance matrix.
%cn = @getnextname("_coefs")
%covn = @getnextname("_cov")
xget(name = {%cn}) BVAR$Bhat
xget(name = {%covn}) BVAR$vcv.Bh

'retrieve fit stats
%llfn = @getnextname("_llf")
%mpostn = @getnextname("_mpost")
%cpostn = @getnextname("_cpost")
xget(name = {%llfn}) BVAR$marg.llf
xget(name = {%mpostn}) BVAR$marg.post
xget(name = {%cpostn}) BVAR$coef.post

'--------------------------------------------------------------------------------------------------------Table Output

'make output spool
%sn = "MSBVAR"
if @isobject(%sn) then
	%sn = @getnextname(%sn)
endif
spool {%sn}

'make output table
%tn = @getnextname("_table")
!cols = @columns({%cn}) +1
table(15,!cols) {%tn}
{%tn}.setwidth(1) 21.0
{%tn}.setwidth(2:!cols) 12
{%tn}.setjust(a1:a8) left
!rowcounter=1
{%tn}(!rowcounter,1) = "Bayesian Vector Autoregression"
!rowcounter = !rowcounter + 1
%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MM")
{%tn}(!rowcounter,1) = %datestring
!rowcounter=!rowcounter+1
%samplestr = "Sample: " + %smpl 
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%obsstr = "Included observations: " + @str(@obssmpl)
{%tn}(!rowcounter,1) = %obsstr
!rowcounter=!rowcounter+1
{%tn}(!rowcounter,1) = "Standard errors in ()"
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

'add column headers - one column for each variable
for !i=1 to @wcount(%vars)
	%var = @word(%vars,!i)
	{%tn}(!rowcounter,!i+1) = @upper(%var)
next
!rowcounter = !rowcounter + 1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

'put in row headers
!temprow = !rowcounter
for !i=1 to @wcount(%vars)
	%var = @word(%vars,!i)
	for !j=1 to @val(%lags)
		%head = @upper(%var) + "(-" + @str(!j) +")"
		{%tn}(!temprow,1) = %head
		!temprow = !temprow+3
	next
next

'put in coefs
!MV = @wcount(%vars)
for !i=1 to !MV
	!temprow = !rowcounter
	for !j=1 to !MV
		for !l=1 to !ML
			!row = (!L-1)*!MV + !j
			{%tn}(!temprow,!i+1) = {%cn}(!row,!i)
			!temprow = !temprow+3
		next
	next
next

'put in SEs
!row= 0
for !i=1 to !MV
	!temprow = !rowcounter+1
	for !j=1 to !MV
		for !l=1 to !ML
			!row= !row+ 1
			{%tn}(!temprow,!i+1) = "(" + @str(@sqrt({%covn}(!row,!row)),"f.5") + ")"
			!temprow = !temprow+3
		next
	next
next

'put in intercept
!rowcounter = !rowcounter + 3*!ml*!mv
{%tn}(!rowcounter,1) = "C"
for !i=1 to !MV
	!row = @rows({%cn})
	{%tn}(!rowcounter,!i+1) = {%cn}(!row,!i)
	!row = !mv*!ml + !i
	{%tn}(!rowcounter+1,!i+1) ="(" + @str(@sqrt({%covn}(!row,!row)),"f.5") + ")"
next
!rowcounter = !rowcounter + 2
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

'add fit results
{%tn}(!rowcounter,1) = "Log-likelihood"
if {%llfn}.@type = "SCALAR" then
	{%tn}(!rowcounter,2) = {%llfn}
else
	{%tn}(!rowcounter,2) = "NA"
endif
{%tn}.setjust(!rowcounter,1) left
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "Data log-posterior"
{%tn}(!rowcounter,2) = {%mpostn}(1,1)
{%tn}.setjust(!rowcounter,1) left
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "Coef log-posterior"
{%tn}(!rowcounter,2) = {%cpostn}(1,1)
{%tn}.setjust(!rowcounter,1) left
!rowcounter = !rowcounter + 1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

'add table to spool
{%sn}.append(name="Est_results") {%tn}
{%sn}.displayname Est_results "Est. Results"


'----------------------------------------------------------------------------------------------------IRFS
if !doirf = 1then
	'make table
	%irftn = @getnextname("_irft")
	table(10,!mv+1) {%irftn}
	!rowcounter = 1
	
	'build command to create impulse responses in R
	%cmd = "IRF<-irf.BVAR(BVAR,nsteps=" + %irfpers + ")"
	xrun %cmd 'run command
	!pers = @val(%irfpers)
	
	%irfn = @getnextname("_irf")
	%graphs = ""
	for !i=1 to !mv  'cycle through each variable
		
		'get IRF for current variable and put it into a matrix.
		%name = %irfn + "_" + @str(!i)
		%cmd = "IRF$mhat[," + @str(!i) + ",]"
		xget(name=%name) {%cmd}
		{%name} = @transpose({%name})
	
		'add table header for current variable
		{%irftn}.setlines(!rowcounter) +d
		!rowcounter = !rowcounter + 1
		%title = "Response of " + @word(%vars,!i) + ":"
		{%irftn}(!rowcounter,1) = %title
		!rowcounter = !rowcounter + 1		
		{%irftn}(!rowcounter,1) = "Period"
		for !j=2 to !mv+1
			{%irftn}(!rowcounter,!j) = @word(%vars,!J-1)
		next
		!rowcounter = !rowcounter + 1
		{%irftn}.setlines(!rowcounter) +d
		!rowcounter = !rowcounter + 1
		
		'add row headers for current variable, and fill in the IRF value from the corresponding matrix
		!tempcounter = !rowcounter
		for !j=1 to !pers
			{%irftn}(!tempcounter,1) = @str(!j,"i")
			for !l=1 to !mv
				{%irftn}(!tempcounter,!l+1) = {%name}(!j,!l)
			next
			!tempcounter = !tempcounter+1
		next
		!rowcounter = !tempcounter
		
		'make graphs
		%tempn = @getnextname("_temp")
		for !j = 1 to !mv
			vector {%tempn} = @columnextract({%name},!j)
			%gvname = @getnextname("_vgraph")
			freeze({%gvname}) {%tempn}.line
			%title = "Response of  " + @upper(@word(%vars,!i)) + " to " + @upper(@word(%vars,!j))
			{%gvname}.addtext(t, font(12))  {%title}
			%graphs = %graphs + " " + %gvname
			d {%tempn}
		next		
		
		d {%name}
	next
	{%irftn}.setlines(!rowcounter) +d
	!rowcounter = !rowcounter + 1
	
	'add table to spool
	{%sn}.append(name="IRF_table") {%irftn}
	{%sn}.displayname IRF_table "IRF Table"
	
	'make parent graph object 
	%gname = @getnextname("_graph")
	graph {%gname}.merge {%graphs}
	{%gname}.addtext(t, font(14)) Impulse Responses
	
	'add graph to spool
	{%sn}.append(name="IRF_graph") {%gname}
	{%sn}.displayname IRF_graph "IRF Graph"
	
else
	{%sn}.options -tree -borders -margins
endif
	
{%sn}.options +displaynames

show {%sn}

'clean up
d(noerr) {%cn} {%covn} {%llfn} {%mpostn} {%cpostn} {%tn} {%irftn} {%graphs} {%gname} {%grp} {%smplser}
smpl {%origsmpl}
if !debug = 0 then
	xclose
endif
